/**************************************************************************

	ns

	script_read.c

	Copyright (C) 1996-2002 by Tomoaki Nakashima. All rights reserved.
		http://www.nakka.com/
		nakka@nakka.com

**************************************************************************/

/**************************************************************************
	Include Files
**************************************************************************/

#include <windows.h>
#include <windowsx.h>
#include <tchar.h>

#include "script.h"
#include "script_string.h"
#include "script_memory.h"


/**************************************************************************
	Define
**************************************************************************/

#define PREP_IMPORT				"import"
#define PREP_LIBRARY			"library"
#define PREP_OPTION				"option"

#define IS_SPACE(c)				(c == ' ' || c == '\t' || c == '\r' || c == '\n')


/**************************************************************************
	Global Variables
**************************************************************************/


/**************************************************************************
	Local Function Prototypes
**************************************************************************/

static long GetFileSerchSize(char *path);
static char *ReadFileBuf(char *path);
static BOOL LoadLibraryFile(SCRIPTINFO *sci, char *FileName);


/******************************************************************************

	GetFileSerchSize

	t@C̃TCY擾

******************************************************************************/

static long GetFileSerchSize(char *path)
{
	WIN32_FIND_DATA FindData;
	HANDLE hFindFile;
#ifdef UNICODE
	TCHAR *tpath;

	tpath = char2tchar(path);
	hFindFile = FindFirstFile(tpath, &FindData);
	mem_free(&tpath);
#else
	hFindFile = FindFirstFile(path, &FindData);
#endif
	if(hFindFile == INVALID_HANDLE_VALUE){
		return -1;
	}
	FindClose(hFindFile);

	if((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0){
		//fBNgł͂Ȃꍇ̓TCYԂ
		return (long)FindData.nFileSizeLow;
	}
	return -1;
}


/******************************************************************************

	ReadFileBuf

	t@Cǂݍ

******************************************************************************/

static char *ReadFileBuf(char *path)
{
	HANDLE hFile;
	DWORD ret;
	char *cBuf;
	long FileSize;
#ifdef UNICODE
	TCHAR *tpath;
#endif

	//t@C̃TCY擾
	FileSize = GetFileSerchSize(path);
	if(FileSize <= 0){
		return NULL;
	}

	//t@CJ
#ifdef UNICODE
	tpath = char2tchar(path);
	hFile = CreateFile(tpath, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	mem_free(&tpath);
#else
	hFile = CreateFile(path, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
#endif
	if(hFile == NULL || hFile == (HANDLE)-1){
		return NULL;
	}
	cBuf = (char *)mem_calloc(FileSize + 1);
	if(cBuf == NULL){
		CloseHandle(hFile);
		return NULL;
	}

	if(ReadFile(hFile, cBuf, FileSize, &ret, NULL) == FALSE){
		mem_free(&cBuf);
		CloseHandle(hFile);
		return NULL;
	}
	CloseHandle(hFile);
	return cBuf;
}


/******************************************************************************

	LoadLibraryFile

	Cuǂݍ

******************************************************************************/

static BOOL LoadLibraryFile(SCRIPTINFO *sci, char *FileName)
{
	SCRIPTINFO *tsci = sci->sci_top;
	EXECINFO ei;
	LIBRARYINFO *lib;
#ifdef UNICODE
	TCHAR *tFileName;
#endif

	if(tsci->lib == NULL){
		lib = tsci->lib = mem_calloc(sizeof(LIBRARYINFO));
	}else{
		for(lib = tsci->lib; lib->next != NULL; lib = lib->next);
		lib = lib->next = mem_calloc(sizeof(LIBRARYINFO));
	}
	if(lib == NULL){
		ZeroMemory(&ei, sizeof(EXECINFO));
		ei.sci = sci;
		Error(&ei, ERR_ALLOC, FileName, FALSE);
		return FALSE;
	}
	//Cu̓ǂݍ
#ifdef UNICODE
	tFileName = char2tchar(FileName);
	lib->hModul = LoadLibrary(tFileName);
	mem_free(&tFileName);
#else
	lib->hModul = LoadLibrary(FileName);
#endif
	if(lib->hModul == NULL){
		ZeroMemory(&ei, sizeof(EXECINFO));
		ei.sci = sci;
		Error(&ei, ERR_LIBRARY, FileName, FALSE);
		return FALSE;
	}
	return TRUE;
}


/******************************************************************************

	ReadScriptFile

	XNvgt@Cǂݍ

******************************************************************************/

BOOL ReadScriptFile(SCRIPTINFO *sci, char *path, char *FileName)
{
	SCRIPTINFO *csci = sci;
	EXECINFO ei;
	char fname[BUF_SIZE];
	char fpath[BUF_SIZE];
	char *name;
	char *p, *r;
	int i = 0;

	p = str_cpy(fpath, path);
	if(*path != '\0' && *(path + tstrlen(path) - 1) != '\\'){
		p = str_cpy(p, "\\");
	}

	for(r = FileName; *r != '\0' && *r != '.'; r++);
	name = alloc_copy_n(FileName, r - FileName + 1);

	//Xg̍쐬
	while(csci->next != NULL){
		csci = csci->next;
		if(csci->name != NULL && str_cmp_i(name, csci->name) == 0 &&
			csci->path != NULL && str_cmp_i(fpath, csci->path) == 0){
			mem_free(&name);
			return TRUE;
		}
	}
	csci = csci->next = mem_alloc(sizeof(SCRIPTINFO));
	if(csci == NULL){
		ZeroMemory(&ei, sizeof(EXECINFO));
		ei.sci = sci;
		Error(&ei, ERR_ALLOC, fname, FALSE);
		return FALSE;
	}
	CopyMemory(csci, sci, sizeof(SCRIPTINFO));
	csci->name = alloc_copy(name);
	csci->path = alloc_copy(fpath);
	csci->buf = NULL;
	csci->fi = NULL;
	csci->vi = NULL;
	csci->lib = NULL;
	csci->next = NULL;

	mem_free(&name);

	//t@C̓ǂݍ
	p = str_cpy(fname, fpath);
	p = str_cpy(p, FileName);

	if(GetFileSerchSize(fname) == 0){
		return TRUE;
	}

	p = csci->buf = ReadFileBuf(fname);
	if(csci->buf == NULL){
		ZeroMemory(&ei, sizeof(EXECINFO));
		ei.sci = csci;
		Error(&ei, ERR_FILEOPEN, fname, FALSE);
		return FALSE;
	}

	//\
	ZeroMemory(&ei, sizeof(EXECINFO));
	ei.Name = name;
	ei.sci = csci;
	csci->tk = ParseSentence(&ei, p, 0);
	return TRUE;
}


/******************************************************************************

	ReadScriptFiles

	XNvgt@Cēǂݍ

******************************************************************************/

BOOL ReadScriptFiles(SCRIPTINFO *sci, char *path, char *FileName)
{
	WIN32_FIND_DATA FindData;
	HANDLE hFindFile;
	char buf[BUF_SIZE];
	char sPath[BUF_SIZE];
	char *p, *r;
#ifdef UNICODE
	TCHAR *tbuf;
	char *cfile;
#endif

	if(tstrlen(path) + tstrlen(FileName) >= BUF_SIZE){
		return FALSE;
	}

	p = str_cpy(buf, path);
	p = str_cpy(p, FileName);
	for(p = r = buf; *p != '\0'; p++){
		if(IsDBCSLeadByte(*p) == TRUE && *(p + 1) != '\0'){
			p++;
		}else if(*p == '\\'){
			r = p + 1;
		}
	}
	str_cpy_n(sPath, buf, r - buf + 1);

#ifdef UNICODE
	tbuf = char2tchar(buf);
	hFindFile = FindFirstFile(tbuf, &FindData);
	mem_free(&tbuf);
#else
	hFindFile = FindFirstFile(buf, &FindData);
#endif
	if(hFindFile == INVALID_HANDLE_VALUE){
		return FALSE;
	}

	do{
		if((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0){
#ifdef UNICODE
			cfile = tchar2char(FindData.cFileName);
			ReadScriptFile(sci, sPath, cfile);
			mem_free(&cfile);
#else
			ReadScriptFile(sci, sPath, FindData.cFileName);
#endif
		}
	} while(FindNextFile(hFindFile, &FindData) == TRUE);

	FindClose(hFindFile);
	return TRUE;
}


/******************************************************************************

	Preprocessor

	vvZbT

******************************************************************************/

char *Preprocessor(SCRIPTINFO *sci, char *path, char *p)
{
	EXECINFO ei;
	char *str;
	char *r, *s, *t;

	for(t = p; IS_SPACE(*t); t++);
	for(; *p != '\0' && *p != '('; p++);
	if(*p == '\0'){
		ZeroMemory(&ei, sizeof(EXECINFO));
		ei.sci = sci;
		Error(&ei, ERR_SENTENCE, t, FALSE);
		return NULL;
	}
	r = get_pair_brace(p);
	if(r == NULL || *r == '\0'){
		ZeroMemory(&ei, sizeof(EXECINFO));
		ei.sci = sci;
		Error(&ei, ERR_BRACE, t, FALSE);
		return NULL;
	}
	for(p++; IS_SPACE(*p); p++);
	str = alloc_copy_n(p, r - p + 1);
	if(str == NULL){
		ZeroMemory(&ei, sizeof(EXECINFO));
		ei.sci = sci;
		Error(&ei, ERR_ALLOC, t, FALSE);
		return NULL;
	}
	if(*str == '\"'){
		s = str_skip(str);
		*s = '\0';
		tstrcpy(str, str + 1);
	}

	if(str_cmp_ni(t, PREP_IMPORT, tstrlen(PREP_IMPORT)) == 0){
		//XNvgt@C̃C|[g
		ReadScriptFiles(sci->sci_top, path, str);

	}else if(str_cmp_ni(t, PREP_LIBRARY, tstrlen(PREP_LIBRARY)) == 0){
		//Cu̓ǂݍ
		LoadLibraryFile(sci, str);

	}else if(str_cmp_ni(t, PREP_OPTION, tstrlen(PREP_OPTION)) == 0){
		//IvV
		if(str_cmp_i(str, "explicit") == 0){
			//Iȕϐ錾̋
			sci->explicit_val = TRUE;
		}

	}else{
		//sȎʎq
		ZeroMemory(&ei, sizeof(EXECINFO));
		ei.sci = sci;
		Error(&ei, ERR_SENTENCE, t, FALSE);
		return NULL;
	}
	mem_free(&str);
	return (r + 1);
}
/* End of source */
