/**************************************************************************

	ns

	script_utility.c

	Copyright (C) 1996-2002 by Tomoaki Nakashima. All rights reserved.
		http://www.nakka.com/
		nakka@nakka.com

**************************************************************************/

/**************************************************************************
	Include Files
**************************************************************************/

#include <windows.h>

#include "script.h"
#include "script_memory.h"
#include "script_string.h"


/**************************************************************************
	Define
**************************************************************************/

/**************************************************************************
	Global Variables
**************************************************************************/

static LCID lcid;


/**************************************************************************
	Local Function Prototypes
**************************************************************************/


/******************************************************************************

	IntToVariable

	lϐɕϊ

******************************************************************************/

VALUEINFO *IntToVariable(char *name, const int i)
{
	VALUEINFO *vi;

	vi = AllocValue();
	if(vi == NULL){
		return NULL;
	}
	vi->Name = alloc_copy(name);
	vi->name_hash = str2hash(name);
	vi->v->u.iValue = i;
	vi->v->Type = TYPE_INTEGER;
	return vi;
}


/******************************************************************************

	VariableToInt

	ϐ𐔒lɕϊ

******************************************************************************/

int VariableToInt(VALUEINFO *vi)
{
	if(vi == NULL || vi->v == NULL){
		return 0;
	}
	return GetValueInt(vi->v);
}


/******************************************************************************

	StringToVariable

	ϐɕϊ

******************************************************************************/

VALUEINFO *StringToVariable(char *name, const char *buf)
{
	VALUEINFO *vi;

	vi = AllocValue();
	if(vi == NULL){
		return NULL;
	}
	vi->Name = alloc_copy(name);
	vi->name_hash = str2hash(name);
	vi->v->u.Value = alloc_copy(buf);
	vi->v->Type = TYPE_STRING;
	return vi;
}


/******************************************************************************

	TStringToVariable

	ϐɕϊ (TCHAR)

******************************************************************************/

VALUEINFO *TStringToVariable(char *name, const TCHAR *buf)
{
	VALUEINFO *vi;

	vi = AllocValue();
	if(vi == NULL){
		return NULL;
	}
	vi->Name = alloc_copy(name);
	vi->name_hash = str2hash(name);
	vi->v->u.Value = tchar2char(buf);
	vi->v->Type = TYPE_STRING;
	return vi;
}


/******************************************************************************

	VariableToString

	ϐ𕶎ɕϊ

******************************************************************************/

char *VariableToString(VALUEINFO *vi)
{
	if(vi == NULL || vi->v == NULL){
		return alloc_copy("");
	}
	return GetValueString(vi->v);
}


/******************************************************************************

	VariableToTString

	ϐ𕶎ɕϊ (TCHAR)

******************************************************************************/

TCHAR *VariableToTString(VALUEINFO *vi)
{
	if(vi == NULL || vi->v == NULL){
		return char2tchar("");
	}
	return TGetValueString(vi->v);
}


/******************************************************************************

	ByteToHash

	nbVɕϊ

******************************************************************************/

BOOL ByteToHash(EXECINFO *ei, VALUEINFO *pvi, char *buf, int len)
{
	VALUEINFO *vi = pvi;
	int i = 0;

	if(pvi->v->Global == TRUE && ei != NULL) GlobalWait(ei->sci->sci_top);

	if(vi->vi == NULL){
		vi = vi->vi = AllocValue();
		vi->v->Global = pvi->v->Global;
	}else{
		vi = vi->vi;
	}
	vi->v->u.iValue = (unsigned char)*(buf + i);
	vi->v->Type = TYPE_INTEGER;

	for(i = 1; i < len; i++){
		if(vi->next == NULL){
			vi = vi->next = AllocValue();
			vi->v->Global = pvi->v->Global;
		}else{
			vi = vi->next;
		}
		vi->v->u.iValue = (unsigned char)*(buf + i);
		vi->v->Type = TYPE_INTEGER;
	}
	if(pvi->v->Global == TRUE && ei != NULL) GlobalUnWait(ei->sci->sci_top);
	return TRUE;
}


/******************************************************************************

	HashToByte

	nbV𕶎ɕϊ

******************************************************************************/

int HashToByte(VALUEINFO *pvi, char **ret)
{
	VALUEINFO *vi;
	char *p;
	int len;

	if(pvi == NULL){
		*ret = (char *)alloc_copy("");
		return 0;
	}
	for(len = 0, vi = pvi->vi; vi != NULL; vi = vi->next, len++);
	p = *ret = mem_alloc(len);
	if(p == NULL){
		*ret = (char *)alloc_copy("");
		return 0;
	}

	for(vi = pvi->vi; vi != NULL; vi = vi->next){
		*(p++) = (char)((vi->v->Type == TYPE_INTEGER) ? vi->v->u.iValue : *vi->v->u.Value);
	}
	return len;
}


/******************************************************************************

	VariableToSystemtime

	ϐSYSTEMTIMEɓWJ

******************************************************************************/

BOOL VariableToSystemtime(SYSTEMTIME *st, VALUEINFO *param)
{
	VALUEINFO *vi;

	ZeroMemory(st, sizeof(st));

	for(vi = param->vi; vi != NULL; vi = vi->next){
		if(vi->Name == NULL){
			continue;
		}
		if(tstrcmp("year", vi->Name) == 0){
			st->wYear = (WORD)VariableToInt(vi);
		}else if(tstrcmp("month", vi->Name) == 0){
			st->wMonth = (WORD)VariableToInt(vi);
		}else if(tstrcmp("week", vi->Name) == 0){
			st->wDayOfWeek = (WORD)VariableToInt(vi);
		}else if(tstrcmp("day", vi->Name) == 0){
			st->wDay = (WORD)VariableToInt(vi);
		}else if(tstrcmp("hour", vi->Name) == 0){
			st->wHour = (WORD)VariableToInt(vi);
		}else if(tstrcmp("minute", vi->Name) == 0){
			st->wMinute = (WORD)VariableToInt(vi);
		}else if(tstrcmp("second", vi->Name) == 0){
			st->wSecond = (WORD)VariableToInt(vi);
		}else if(tstrcmp("milliseconds", vi->Name) == 0){
			st->wMilliseconds = (WORD)VariableToInt(vi);
		}
	}
	return TRUE;
}


/******************************************************************************

	SystemtimeToVariable

	ϐSYSTEMTIMEɓWJ

******************************************************************************/

BOOL SystemtimeToVariable(const SYSTEMTIME *st, VALUEINFO *ret)
{
	VALUEINFO *vi;

	vi = ret->vi = IntToVariable("year", st->wYear);
	vi = vi->next = IntToVariable("month", st->wMonth);
	vi = vi->next = IntToVariable("week", st->wDayOfWeek);
	vi = vi->next = IntToVariable("day", st->wDay);
	vi = vi->next = IntToVariable("hour", st->wHour);
	vi = vi->next = IntToVariable("minute", st->wMinute);
	vi = vi->next = IntToVariable("second", st->wSecond);
	vi = vi->next = IntToVariable("milliseconds", st->wMilliseconds);
	ret->Hash = TRUE;
	return TRUE;
}


/******************************************************************************

	GetLocale

	P[̎擾

******************************************************************************/

LCID GetLocale(void)
{
	if(lcid == 0){
		lcid = GetUserDefaultLCID();
	}
	return lcid;
}


/******************************************************************************

	SetLocale

	P[̐ݒ

******************************************************************************/

BOOL SetLocale(LCID li)
{
	lcid = li;
	return TRUE;
}
/* End of source */
